@@ -14,7 +14,7 @@
  * The Original Code is Enigmail.
  *
  * The Initial Developer of the Original Code is Patrick Brunschwig.
- * Portions created by Patrick Brunschwig <patrick@mozilla-enigmail.org> are
+ * Portions created by Patrick Brunschwig <patrick@enigmail.net> are
  * Copyright (C) 2004 Patrick Brunschwig. All Rights Reserved.
  *
  * Contributor(s):
@@ -39,6 +39,9 @@ Components.utils.import("resource://enigmail/keyManagement.jsm");
 // Initialize enigmailCommon
 EnigInitCommon("enigmailKeyManager");
 
+const Cc = Components.classes;
+const Ci = Components.interfaces;
+
 const INPUT = 0;
 const RESULT = 1;
 
@@ -52,9 +55,6 @@ var gEnigLastSelectedKeys = null;
 var gKeySortList = null;
 var gEnigIpcRequest = null;
 var gEnigCallbackFunc = null;
-var gClearButton = null;
-var gFilterBox = null;
-var gSearchTimer = null;
 var gSearchInput = null;
 var gShowAllKeysElement = null;
 var gTreeChildren = null;
@@ -65,8 +65,6 @@ var gShowOthersKeys = null;
 function enigmailKeyManagerLoad() {
   DEBUG_LOG("enigmailKeyManager.js: enigmailKeyManagerLoad\n");
   gUserList = document.getElementById("pgpKeyList");
-  gFilterBox = document.getElementById("filterKey");
-  gClearButton = document.getElementById("clearFilter");
   gSearchInput = document.getElementById("filterKey");
   gShowAllKeysElement = document.getElementById("showAllKeys");
   gTreeChildren = document.getElementById("pgpKeyListChildren");
@@ -643,7 +641,7 @@ function enigCreateKeyMsg() {
   var tmpDir=EnigGetTempDir();
 
   try {
-    var tmpFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(EnigGetLocalFileApi());
+    var tmpFile = Cc[ENIG_LOCAL_FILE_CONTRACTID].createInstance(EnigGetLocalFileApi());
     tmpFile.initWithPath(tmpDir);
     if (!(tmpFile.isDirectory() && tmpFile.isWritable())) {
       EnigAlert(EnigGetString("noTempDir"));
@@ -652,7 +650,7 @@ function enigCreateKeyMsg() {
   }
   catch (ex) {}
   tmpFile.append("key.asc");
-  tmpFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
+  tmpFile.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
 
   // save file
   var exitCodeObj= {};
@@ -664,9 +662,9 @@ function enigCreateKeyMsg() {
   }
 
   // create attachment
-  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+  var ioServ = Cc[ENIG_IOSERVICE_CONTRACTID].getService(Ci.nsIIOService);
   var tmpFileURI = ioServ.newFileURI(tmpFile);
-  var keyAttachment = Components.classes["@mozilla.org/messengercompose/attachment;1"].createInstance(Components.interfaces.nsIMsgAttachment);
+  var keyAttachment = Cc["@mozilla.org/messengercompose/attachment;1"].createInstance(Ci.nsIMsgAttachment);
   keyAttachment.url = tmpFileURI.spec;
   if (keyList.length == 1) {
     keyAttachment.name = "0x"+keyList[0].substr(-8,8)+".asc";
@@ -678,18 +676,18 @@ function enigCreateKeyMsg() {
   keyAttachment.contentType = "application/pgp-keys";
 
   // create Msg
-  var msgCompFields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
+  var msgCompFields = Cc["@mozilla.org/messengercompose/composefields;1"].createInstance(Ci.nsIMsgCompFields);
   msgCompFields.addAttachment(keyAttachment);
 
-  var acctManager = Components.classes["@mozilla.org/messenger/account-manager;1"].createInstance(Components.interfaces.nsIMsgAccountManager);
+  var acctManager = Cc["@mozilla.org/messenger/account-manager;1"].createInstance(Ci.nsIMsgAccountManager);
 
-  var msgCompSvc = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);
+  var msgCompSvc = Cc["@mozilla.org/messengercompose;1"].getService(Ci.nsIMsgComposeService);
 
-  var msgCompParam = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
+  var msgCompParam = Cc["@mozilla.org/messengercompose/composeparams;1"].createInstance(Ci.nsIMsgComposeParams);
   msgCompParam.composeFields = msgCompFields;
   msgCompParam.identity = acctManager.defaultAccount.defaultIdentity;
-  msgCompParam.type = Components.interfaces.nsIMsgCompType.New;
-  msgCompParam.format = Components.interfaces.nsIMsgCompFormat.Default;
+  msgCompParam.type = Ci.nsIMsgCompType.New;
+  msgCompParam.format = Ci.nsIMsgCompFormat.Default;
   msgCompParam.originalMsgURI = "";
   msgCompSvc.OpenComposeWindowWithParams("", msgCompParam);
 }
@@ -728,18 +726,18 @@ function createNewMail() {
   }
 
   // create Msg
-  var msgCompFields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
+  var msgCompFields = Cc["@mozilla.org/messengercompose/composefields;1"].createInstance(Ci.nsIMsgCompFields);
   msgCompFields.to = addresses.join(", ");
 
-  var acctManager = Components.classes["@mozilla.org/messenger/account-manager;1"].createInstance(Components.interfaces.nsIMsgAccountManager);
+  var acctManager = Cc["@mozilla.org/messenger/account-manager;1"].createInstance(Ci.nsIMsgAccountManager);
 
-  var msgCompSvc = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);
+  var msgCompSvc = Cc["@mozilla.org/messengercompose;1"].getService(Ci.nsIMsgComposeService);
 
-  var msgCompParam = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
+  var msgCompParam = Cc["@mozilla.org/messengercompose/composeparams;1"].createInstance(Ci.nsIMsgComposeParams);
   msgCompParam.composeFields = msgCompFields;
   msgCompParam.identity = acctManager.defaultAccount.defaultIdentity;
-  msgCompParam.type = Components.interfaces.nsIMsgCompType.New;
-  msgCompParam.format = Components.interfaces.nsIMsgCompFormat.Default;
+  msgCompParam.type = Ci.nsIMsgCompType.New;
+  msgCompParam.format = Ci.nsIMsgCompFormat.Default;
   msgCompParam.originalMsgURI = "";
   msgCompSvc.OpenComposeWindowWithParams("", msgCompParam);
 }
@@ -761,6 +759,23 @@ function enigEditKeyTrust() {
   }
 }
 
+function enigEditKeyExpiry() {
+
+  var keyList = enigmailGetSelectedKeys();
+  if (keyList.length==0) {
+    EnigAlert(EnigGetString("noKeySelected"));
+    return;
+  }
+  var userIdList = [];
+  for (var i=0; i < keyList.length; i++) {
+    userIdList.push(gKeyList[keyList[i]].userId);
+  }
+
+  if (EnigEditKeyExpiry(userIdList, keyList)) {
+    enigmailRefreshKeys();
+  }
+}
+
 
 function enigSignKey() {
   var keyList = enigmailGetSelectedKeys();
@@ -927,16 +942,16 @@ function enigmailChangePwd() {
 function enigGetClipboard() {
   DEBUG_LOG("enigmailKeyManager.js: enigGetClipboard:\n");
   var cBoardContent = "";
-  var clipBoard = Components.classes[ENIG_CLIPBOARD_CONTRACTID].getService(Components.interfaces.nsIClipboard);
+  var clipBoard = Cc[ENIG_CLIPBOARD_CONTRACTID].getService(Ci.nsIClipboard);
   try {
-    var transferable = Components.classes[ENIG_TRANSFERABLE_CONTRACTID].createInstance(Components.interfaces.nsITransferable);
+    var transferable = Cc[ENIG_TRANSFERABLE_CONTRACTID].createInstance(Ci.nsITransferable);
     transferable.addDataFlavor("text/unicode");
     clipBoard.getData(transferable, clipBoard.kGlobalClipboard);
     var flavour = {};
     var data = {};
     var length = {};
     transferable.getAnyTransferData(flavour, data, length);
-    cBoardContent=data.value.QueryInterface(Components.interfaces.nsISupportsString).data;
+    cBoardContent=data.value.QueryInterface(Ci.nsISupportsString).data;
     DEBUG_LOG("enigmailKeyManager.js: enigGetClipboard: got data\n");
   }
   catch(ex) {}
@@ -976,9 +991,9 @@ function enigmailCopyToClipbrd() {
     EnigAlert(EnigGetString("copyToClipbrdFailed")+"\n\n"+errorMsgObj.value);
     return;
   }
-  var clipBoard = Components.classes[ENIG_CLIPBOARD_CONTRACTID].getService(Components.interfaces.nsIClipboard);
+  var clipBoard = Cc[ENIG_CLIPBOARD_CONTRACTID].getService(Ci.nsIClipboard);
   try {
-    clipBoardHlp = Components.classes[ENIG_CLIPBOARD_HELPER_CONTRACTID].getService(Components.interfaces.nsIClipboardHelper);
+    clipBoardHlp = Cc[ENIG_CLIPBOARD_HELPER_CONTRACTID].getService(Ci.nsIClipboardHelper);
     clipBoardHlp.copyStringToClipboard(keyData, clipBoard.kGlobalClipboard);
     if (clipBoard.supportsSelectionClipboard()) {
       clipBoardHlp.copyStringToClipboard(keyData, clipBoard.kSelectionClipboard);
@@ -1043,6 +1058,94 @@ function enigmailRefreshAllKeys() {
   if (doIt) enigmailKeyServerAcess(nsIEnigmail.REFRESH_KEY, enigmailReceiveKeyCb);
 }
 
+// Iterate through contact emails and download them
+function enigmailDowloadContactKeysEngine() {
+  let abManager = Cc["@mozilla.org/abmanager;1"].getService(Ci.nsIAbManager);
+
+  let allAddressBooks = abManager.directories;
+  let emails = new Array();
+
+  while (allAddressBooks.hasMoreElements()) {
+    let addressBook = allAddressBooks.getNext().QueryInterface(Ci.nsIAbDirectory);
+
+    if (addressBook instanceof Ci.nsIAbDirectory) { // or nsIAbItem or nsIAbCollection
+      // ask for confirmation for each address book:
+      var doIt = EnigmailCommon.confirmDlg(window,
+                   EnigGetString("downloadContactsKeys.importFrom", addressBook.dirName),
+                   EnigGetString("dlgYes"),
+                   EnigGetString("dlg.button.skip"));
+      if (!doIt) {
+        continue;  // SKIP this address book
+      }
+
+      let allChildCards = addressBook.childCards;
+
+      while (allChildCards.hasMoreElements()) {
+
+        let card = allChildCards.getNext().QueryInterface(Ci.nsIAbCard);
+
+        try {
+          let email = card.getPropertyAsAString("PrimaryEmail");
+          if (email && email.indexOf("@")>=0) {
+            emails.push(email);
+          }
+        }
+        catch (e) {}
+
+        try {
+          let email = card.getPropertyAsAString("SecondEmail");
+          if (email && email.indexOf("@")>=0) {
+            emails.push(email);
+          }
+        }
+        catch (e) {}
+
+      }
+    }
+  }
+
+  // list of emails might be emoty here, in which case we do nothing
+  if (emails.length <= 0) {
+    return;
+  }
+
+  // sort the e-mail array
+  emails.sort();
+
+  //remove duplicates
+  var i = 0;
+  while (i<emails.length-1) {
+    if (emails[i] == emails[i+1]) {
+      emails.splice(i,1);
+    }
+    else {
+      i = i + 1;
+    }
+  }
+
+  var inputObj = {
+    searchList : emails
+  }
+  var resultObj = new Object();
+
+  EnigmailFuncs.downloadKeys(window, inputObj, resultObj);
+
+  if (resultObj.importedKeys > 0) {
+    enigmailRefreshKeys();
+  }
+}
+
+function enigmailDownloadContactKeys() {
+
+  var doIt = EnigmailCommon.confirmPref(window,
+    EnigGetString("downloadContactsKeys.warn"),
+    "warnDownloadContactKeys",
+    EnigGetString("dlg.button.continue"),
+    EnigGetString("dlg.button.cancel"));
+
+  if (doIt) enigmailDowloadContactKeysEngine();
+}
+
 function displayResult(arrayOfMsgText) {
   EnigLongAlert(arrayOfMsgText.join("\n"));
 }
@@ -1088,39 +1191,13 @@ function addToPRRule() {
 // ----- key filtering functionality  -----
 //
 
-function onSearchInput(returnKeyHit)
-{
-  if (gSearchTimer) {
-    gSearchTimer.cancel();
-    gSearchTimer = null;
-  }
 
-  // only select the text when the return key was hit
-  if (returnKeyHit) {
-    gSearchInput.select();
-    onEnterInSearchBar();
-  }
-  else {
-    gSearchTimer = EnigmailCommon.setTimeout(onEnterInSearchBar, 600);
-  }
-}
-
-function onSearchKeyPress(event)
-{
-  // 13 == return
-  if (event && event.keyCode == 13) {
-    event.stopPropagation(); // make sure the dialog is not closed...
-    onSearchInput(true);
-  }
-}
-
-function onEnterInSearchBar() {
+function onSearchInput() {
    if (gSearchInput.value == "")
    {
      onResetFilter();
      return;
    }
-   gClearButton.setAttribute("disabled", false);
    enigApplyFilter();
 }
 
@@ -1129,9 +1206,8 @@ function getFirstNode() {
 }
 
 function onResetFilter() {
-  gFilterBox.value="";
+  gSearchInput.value="";
   showOrHideAllKeys();
-  gClearButton.setAttribute("disabled", true);
 }
 
 function enigmailToggleShowAll() {
@@ -1173,7 +1249,7 @@ function determineHiddenKeys(keyObj, showInvalidKeys, showUntrustedKeys, showOth
   var show = true;
 
   const INVALID_KEYS = "ierdD";
-  const UNTRUSTED_KEYS = "n";
+  const UNTRUSTED_KEYS = "n-";
 
   if ((!showInvalidKeys) && INVALID_KEYS.indexOf(EnigGetTrustCode(keyObj))>=0) show = false;
   if ((!showUntrustedKeys) && UNTRUSTED_KEYS.indexOf(keyObj.ownerTrust)>=0) show = false;
@@ -1291,27 +1367,27 @@ function getSortDirection() {
 
 function sortTree(column) {
 
-	var columnName;
-	var order = getSortDirection();
+  var columnName;
+  var order = getSortDirection();
 
-	//if the column is passed and it's already sorted by that column, reverse sort
-	if (column) {
-		columnName = column.id;
-		if (gUserList.getAttribute("sortResource") == columnName) {
-			order *= -1;
-		}
-		else {
-  		document.getElementById(gUserList.getAttribute("sortResource")).removeAttribute("sortDirection");
-  		order = 1;
-		}
-	} else {
-		columnName = gUserList.getAttribute("sortResource");
-	}
-	gUserList.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
-	gUserList.setAttribute("sortResource", columnName);
-	document.getElementById(columnName).setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
+  //if the column is passed and it's already sorted by that column, reverse sort
+  if (column) {
+    columnName = column.id;
+    if (gUserList.getAttribute("sortResource") == columnName) {
+      order *= -1;
+    }
+    else {
+      document.getElementById(gUserList.getAttribute("sortResource")).removeAttribute("sortDirection");
+      order = 1;
+    }
+  } else {
+    columnName = gUserList.getAttribute("sortResource");
+  }
+  gUserList.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
+  gUserList.setAttribute("sortResource", columnName);
+  document.getElementById(columnName).setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
   enigmailClearTree();
-	enigmailBuildList(false);
+  enigmailBuildList(false);
   enigApplyFilter();
 }
 
