@@ -18,7 +18,7 @@
  * Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
  *
  * Contributor(s):
- *   Patrick Brunschwig <patrick@mozilla-enigmail.org>
+ *   Patrick Brunschwig <patrick@enigmail.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -165,6 +165,12 @@ Enigmail.msg = {
     };
 
     top.controllers.appendController(treeController);
+
+    EnigmailCommon.initPrefService();
+    if (EnigmailCommon.getPref("configuredVersion") == "") {
+      EnigmailCommon.setPref("configuredVersion", EnigmailCommon.getVersion());
+      EnigmailFuncs.openSetupWizard(window);
+    }
   },
 
   viewSecurityInfo: function (event, displaySmimeMsg)
@@ -537,6 +543,7 @@ Enigmail.msg = {
 
       var ct = mimePart.headers["content-type"][0];
       if (typeof(ct) == "string") {
+        ct = ct.replace(/[\r\n]/g, " ");
         if (ct.search(/multipart\/signed.*application\/pgp-signature/i) >= 0) {
           resultObj.signed=mimePart.partName;
         }
@@ -703,7 +710,7 @@ Enigmail.msg = {
             Enigmail.msg.verifyEmbeddedMsg(window, mailNewsUrl, msgWindow, msgUriSpec, contentEncoding, event);
           }
           else {
-            var verifier = EnigmailVerify.newVerfier(false, mailNewsUrl);
+            var verifier = EnigmailVerify.newVerifier(false, mailNewsUrl, false);
             verifier.startStreaming(window, msgWindow, msgUriSpec);
 
           }
@@ -1622,7 +1629,7 @@ Enigmail.msg = {
 
         let enableSubpartTreatment=(msigned > 0);
 
-        var verifier = EnigmailVerify.newVerfier(enableSubpartTreatment, callbackArg.mailNewsUrl);
+        var verifier = EnigmailVerify.newVerifier(enableSubpartTreatment, callbackArg.mailNewsUrl, true);
         verifier.verifyData(callbackArg.window, callbackArg.msgWindow, callbackArg.msgUriSpec, callbackArg.data);
 
         return;
@@ -1650,17 +1657,23 @@ Enigmail.msg = {
   handleAttachmentSel: function (actionType)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMessengerOverlay.js: handleAttachmentSel: actionType="+actionType+"\n");
+    var selectedAttachments;
+    var anAttachment;
 
     // Thunderbird
     var contextMenu = document.getElementById('attachmentItemContext');
-    var selectedAttachments = contextMenu.attachments;
 
-    if (! contextMenu) {
+    if (contextMenu) {
+      // Thunderbird
+      selectedAttachments = contextMenu.attachments;
+      anAttachment = selectedAttachments[0];
+    }
+    else {
       // SeaMonkey
       contextMenu = document.getElementById('attachmentListContext');
-      selectedAttachments = attachmentList.selectedItems;
+      selectedAttachments = document.getElementById('attachmentList').selectedItems;
+      anAttachment = selectedAttachments[0].attachment;
     }
-    var anAttachment = selectedAttachments[0];
 
     switch (actionType) {
       case "saveAttachment":
@@ -1750,7 +1763,7 @@ Enigmail.msg = {
     var r = enigmailSvc.verifyAttachment(window, outFile1, outFile2, statusFlagsObj, errorMsgObj);
 
     if (r == 0)
-      EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifiedOK", [ this.getAttachmentName(origAtt) ]));
+      EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifiedOK", [ this.getAttachmentName(origAtt) ]) +"\n\n"+ errorMsgObj.value);
     else
       EnigmailCommon.alert(window, EnigmailCommon.getString("signature.verifyFailed", [ this.getAttachmentName(origAtt) ])+"\n\n"+
         errorMsgObj.value);
@@ -1964,9 +1977,10 @@ Enigmail.msg = {
             Enigmail.msg.loadExternalURL(outFileUri.asciiSpec);
           }
         }
-
-        // open the attachment using an external application
-        Enigmail.msg.loadExternalURL(outFileUri.asciiSpec);
+        else {
+          // open the attachment using an external application
+          Enigmail.msg.loadExternalURL(outFileUri.asciiSpec);
+        }
       }
     }
   },
